# -*- coding: utf-8 -*-


from configobj import ConfigObj
import sys, os, pwd
from pickle import dump, load
from time import sleep
import ConfigParser
#from pyeole.service import service_code
from pyeole.process import system_out, tcpcheck
from pyeole.schedule import load_schedule
from creole.parsedico import parse_dico

from creole import config

BACULA_CONF = "/etc/eole/bacula.conf"

BCONSOLE_CONF = '/etc/bacula/bconsole.conf'

BACULA_SUPPORT = '/var/lib/eole/config/baculasupport.conf'
BACULA_MAIL = '/var/lib/eole/config/baculamail.conf'
BACULA_JOB = '/var/lib/eole/config/baculajobs.conf'
BACULA_CONFIG_TMPL = '/var/lib/creole/baculasupport.conf'
BACULA_CONFIG_MAILTMPL = '/var/lib/creole/baculamail.conf'
BACULA_CONFIG_FILE = '/etc/bacula/baculasupport.conf'
BACULA_CONFIG_JOB = '/etc/bacula/baculaschedule.conf'
BACULA_CONFIG_MAIL = '/etc/bacula/baculamail.conf'
BACULA_CONFIG_JOBSCHED = '/etc/bacula/baculaschedulepost.conf'
BACULA_CONFIG_DIR_XML = '/usr/share/eole/bacula/dicos/'
MOUNT_POINT = '/mnt/sauvegardes'
TEST_MOUNT_FILE = os.path.join(MOUNT_POINT, 'eole_test.txt')

dico = None

def load_dico():
    global dico
    if dico == None:
        dico = parse_dico()

class Disabled(Exception):
    pass

def bacula_active():
    load_dico()
    return dico['activer_bacula'] == 'oui'

def bacula_active_sd():
    load_dico()
    if bacula_active():
        return 'oui' == dico['activer_bacula_sd']
    return False

def bacula_active_dir():
    load_dico()
    if bacula_active():
        return 'oui' == dico['activer_bacula_dir']
    return False

def load_bacula_conf(filename):
    cfgparser = ConfigParser.ConfigParser()
    dic = {}
    cfgparser.read(filename)
    for section in cfgparser.sections():
        dic[section] = eval(cfgparser.get(section, 'val'))[0]
    return dic

def reload_conf(test='both'):
    if test not in ['both', 'sd', 'dir']:
        raise Exception('ERREUR : test doit être "both", "sd" ou "dir')
    if test != 'sd':
        cmd = ['/usr/sbin/bacula-dir', '-t']
        ret, stdout, stderr = system_out(cmd)
        if ret != 0:
            raise Exception('ERREUR : erreur au test de la configuration : {0}, {1}'.format(stdout,stderr))
    if test != 'dir':
        cmd = ['/usr/sbin/bacula-sd', '-t']
        ret, stdout, stderr = system_out(cmd)
        if ret != 0:
            raise Exception('ERREUR : erreur au test de la configuration : {0}, {1}'.format(stdout,stderr))
    run_bacula_cmd('reload')

def _save_bacula_conf(filename, dic):
    fh = file(filename, 'w')
    cfgparser = ConfigParser.ConfigParser()
    for section, val in dic.items():
        cfgparser.add_section(section)
        if type(val) == list:
            cfgparser.set(section, 'val', val)
        else:
            cfgparser.set(section, 'val', [val])
        cfgparser.set(section, 'valprec', [])
        cfgparser.set(section, 'valdefault', [])
    cfgparser.write(fh)
    fh.close()

def mount_bacula_support(usb_test=40):
    #demonte avant de faire le montage
    umount_bacula_support()
    config = ConfigObj(BACULA_CONF)
    USB_MOUNT = config.get('USB_MOUNT', '/bin/mount {0} {1} -o uid={2}')
    DISTANT_LOGIN_MOUNT = config.get('DISTANT_LOGIN_MOUNT', '/bin/mount -t smbfs -o username={0},password={1},ip={2},uid={3} //{4}/{5} {6}')
    DISTANT_MOUNT = config.get('DISTANT_MOUNT', '/bin/mount -t smbfs -o password={0},ip={1},uid={2} //{3}/{4} {5}')

    dic=load_bacula_support()
    if dic['support'] == 'manual':
        return 'Pas de montage pour la configuration manuelle'

    if dic['support'] == 'none':
        raise Exception("ERREUR : bacula n'est pas configuré")

    if not os.path.isdir(MOUNT_POINT):
        os.makedirs(MOUNT_POINT)
    #montage
    #mount as bacula user
    try:
        uid = pwd.getpwnam('bacula').pw_uid
    except:
        uid = 105
    if dic['support'] == 'usb':
        if device_is_mount(dic['usb_path']):
            raise Exception('ERREUR : périphérique {0} déjà monté'.format(dic['usb_path']))
        cmd = USB_MOUNT.format(dic['usb_path'], MOUNT_POINT, uid).split()
        nbtest = usb_test
    elif dic['support'] == 'smb':
        ret = tcpcheck(dic['smb_ip'], '139', 2)
        if not ret:
            ret = tcpcheck(dic['smb_ip'], '445', 2)
            if not ret:
                raise Exception("La machine {0} n'est pas accessible sur les ports 139 et 445".format(dic['smb_ip']))
        if dic['smb_login'] != None:
            cmd = DISTANT_LOGIN_MOUNT.format(dic['smb_login'],
                    dic['smb_password'], dic['smb_ip'], uid,
                    dic['smb_machine'], dic['smb_partage'], MOUNT_POINT).split()
        else:
            cmd = DISTANT_MOUNT.format(dic['smb_password'], dic['smb_ip'], uid,
                    dic['smb_machine'], dic['smb_partage'], MOUNT_POINT).split()
        nbtest = 1
    ret = 1
    for i in range(1, nbtest+1):
        ret, stdout, stderr = system_out(cmd)
        if ret == 0:
            break
        umount_bacula_support()
        print "Problème de montage (essai {0}/{1})".format(i, nbtest)
        sleep(3)

    if ret != 0:
        umount_bacula_support()
        raise Exception('ERREUR : montage impossible : {0}, {1}'.format(stdout,stderr))

    if not os.path.ismount(MOUNT_POINT):
        umount_bacula_support()
        raise Exception('ERREUR : point de montage {0} non monté'.format(MOUNT_POINT))
    return True

def umount_bacula_support(error=False):
    while os.path.ismount(MOUNT_POINT):
        cmd = ['/bin/umount', MOUNT_POINT]
        ret, stdout, stderr = system_out(cmd)
        if ret != 0 and error:
            raise Exception('ERREUR : démontage impossible : {0}, {1}'.format(stdout,stderr))
    return True

def device_is_mount(device):
    if not os.path.exists(device):
        raise Exception("ERREUR : le périphérique {0} n'existe pas".format(device))
    cmd = ['grep', '-q',  '^{0} '.format(device), '/etc/mtab']
    ret, stdout, stderr = system_out(cmd)
    return ret == 0

def run_bacula_cmd(stdin):
    cmd = ['bconsole', '-c', BCONSOLE_CONF]
    return system_out(cmd, stdin=stdin)

def _empty_file(filename):
    if os.path.isfile(filename):
        os.unlink(filename)
    fh = file(filename,'w')
    fh.write('')
    fh.close()

def run_backup(level='Full'):
    if level not in ['Full', 'Incremental', 'Differential']:
        raise Exception('level inconnu')
    ret, stdout, stderr = run_bacula_cmd("run JobSchedulePre Level=Full\n\n")
    if ret != 0:
        raise Exception('ERREUR : de sauvegarde {0} : {1}, {2}'.format(level, stdout, stderr))
    ret, stdout, stderr = run_bacula_cmd("run JobSauvegarde Level={0}\n\n".format(level))
    if ret != 0:
        raise Exception('ERREUR : de sauvegarde {0} : {1}, {2}'.format(level, stdout, stderr))
    #run catalog backup every time
    run_bacula_cmd("run BackupCatalog level=Full\n\n")
    if ret != 0:
        raise Exception('ERREUR : de sauvegarde du catalogue {0} : {1}, {2}'.format(level, stdout, stderr))
    return "Sauvegarde {0} lancée".format(level)

def instance_bacula(tmpl, dest, eol):
    if not os.path.isfile(tmpl):
        raise Exception('Fichier template {0} inexistant'.format(tmpl))
    from creole.creolecat import instance
    xml_dir=[]
    xml_dir.extend(config.eoledirs)
    xml_dir.append(BACULA_CONFIG_DIR_XML)
    xml_dir=','.join(xml_dir)
    eol = [config.configeol, eol]
    instance(tmpl, xml_dir, dest, eol)

# -------------------------------------------------
# -------------- Bacula support -------------------
# -------------------------------------------------
def load_bacula_support():
    if not bacula_active_sd():
        raise Disabled('Bacula sd désactivé')
    if not os.path.isfile(BACULA_SUPPORT):
        return {'support': 'none'}
    return load_bacula_conf(filename=BACULA_SUPPORT)

def save_bacula_support_usb(usb_path):
    if not bacula_active_sd():
        raise Disabled('Bacula sd désactivé')
    if usb_path == None:
        raise Exception('Chemin usb inconnu')
    dic = {'usb_path': usb_path, 'support': 'usb'}
    _save_bacula_conf(filename=BACULA_SUPPORT, dic=dic)
    apply_bacula_support()

def save_bacula_support_smb(smb_machine, smb_ip,
        smb_partage, smb_login=None, smb_password=None):
    if not bacula_active_sd():
        raise Disabled('Bacula sd désactivé')

    if smb_machine == None or smb_ip  == None or smb_partage == None:
        raise Exception('Nom machine, IP ou nom du partage inconnu')
    dic = {'support': 'smb'}
    if smb_login != None and smb_password == None:
        raise Exception('Login spécifié mais password absent')
    if smb_login == None and smb_password != None:
        raise Exception('Password spécifié mais login absent')

    dic['smb_machine'] = smb_machine
    dic['smb_ip'] = smb_ip
    dic['smb_partage'] = smb_partage
    dic['smb_login'] = smb_login
    dic['smb_password'] = smb_password
    _save_bacula_conf(filename=BACULA_SUPPORT, dic=dic)
    apply_bacula_support()

def save_bacula_support_manual():
    if not bacula_active_sd():
        raise Disabled('Bacula sd désactivé')
    dic = {'support': 'manual'}
    _save_bacula_conf(filename=BACULA_SUPPORT, dic=dic)
    apply_bacula_support()

def apply_bacula_support(load=True):
    if not bacula_active_sd():
        if os.path.isfile(BACULA_CONFIG_FILE):
            os.unlink(BACULA_CONFIG_FILE)
        raise Disabled('Bacula sd désactivé')
    if load_bacula_support()['support'] == 'none':
        print "bacula n'est pas configuré"
        _empty_file(BACULA_CONFIG_FILE)
        #cree un fichier eol minimal
        dic = {'support': 'none'}
        _save_bacula_conf(filename=BACULA_SUPPORT, dic=dic)
    instance_bacula(BACULA_CONFIG_TMPL, BACULA_CONFIG_FILE, BACULA_SUPPORT)
    if load:
        reload_conf(test='sd')

def test_bacula_support():
    if not bacula_active_sd():
        raise Disabled('Bacula sd désactivé')
    if os.path.ismount(MOUNT_POINT):
        raise Exception('ERREUR : point de montage {0} déjà occupé'.format(MOUNT_POINT))

    try:
        mount_bacula_support(usb_test=2)
    except Exception, e:
        raise Exception(e)

    if os.path.isfile(TEST_MOUNT_FILE):
        umount_bacula_support()
        raise Exception('ERREUR : le fichier test {0} existe déjà'.format(TEST_MOUNT_FILE))

    #ecriture du fichier
    try:
        fh = file(TEST_MOUNT_FILE, 'w')
        fh.write('test')
        fh.close()
        os.unlink(TEST_MOUNT_FILE)
    except:
        umount_bacula_support()
        raise Exception('ERREUR : écriture impossible')

    #demontage
    umount_bacula_support(error=True)

    return "Tests de montage OK"

# -------------------------------------------------
# ----------------- Bacula mail -------------------
# -------------------------------------------------
def load_bacula_mail():
    if not bacula_active_dir():
        raise Disabled('Bacula dir désactivé')
    if not os.path.isfile(BACULA_MAIL):
        return False
    return load_bacula_conf(filename=BACULA_MAIL)

def save_bacula_mail(mail_ok=None, mail_error=None):
    if not bacula_active_dir():
        raise Disabled('Bacula dir désactivé')
    dic = {}
    if mail_ok != None:
        dic['mail_ok'] = mail_ok
    if mail_error != None:
        dic['mail_error'] = mail_error
    _save_bacula_conf(filename=BACULA_MAIL, dic=dic)

def apply_bacula_mail(load=True):
    if not bacula_active_dir():
        if os.path.isfile(BACULA_CONFIG_MAIL):
            os.unlink(BACULA_CONFIG_MAIL)
        raise Disabled('Bacula dir désactivé')
    if load_bacula_mail() == False:
        print "les mails bacula ne sont pas configurés"
        _empty_file(BACULA_CONFIG_MAIL)
        return
    instance_bacula(BACULA_CONFIG_MAILTMPL, BACULA_CONFIG_MAIL, BACULA_MAIL)
    if load:
        reload_conf(test='dir')


# -------------------------------------------------
# ------------------ Bacula job -------------------
# -------------------------------------------------

DAY_TO_STRING = {1: 'dimanche au lundi', 2: 'lundi au mardi',
        3: 'mardi au mercredi', 4: 'mercredi au jeudi', 5: 'jeudi au vendredi',
        6: 'vendredi au samedi', 7: 'samedi au dimanche'}

DAY_TO_BACULA = {1: 'mon', 2: 'tue', 3: 'wed', 4: 'thu', 5: 'fri', 6: 'sat', 7: 'sun'}
LEVEL_TO_FR = {'Incremental':'Incrémentale', 'Full':'Totale', 'Differential': 'Différentielle'}

def load_bacula_jobs(check_active=True):
    """
    check_active: don't load dicos to check if bacula-dir is activate
    """
    if check_active and not bacula_active_dir():
        raise Disabled('Bacula dir désactivé')
    try:
        dicosched = load(file(BACULA_JOB, 'r'))
    except:
        dicosched = []
    return dicosched

def save_bacula_job(dic):
    if not bacula_active_dir():
        raise Disabled('Bacula dir désactivé')
    dump(dic,file(BACULA_JOB, 'w'))

def add_job(job, level, day, hour, end_day=None):
    """
    job: weekly, daily or monthly
    level: Full/Incremental/Differential
    day: day in week
    hour: hour of job
    end_day: used only for daily job
    """
    if not bacula_active_dir():
        raise Disabled('Bacula dir désactivé')
    def raise_if_conflict(day, index):
        raise Exception('Sauvegarde de la nuit du {0} entre en conflit avec la sauvegarde {1}'.format(DAY_TO_STRING[int(day)], index))

    if level not in ['Full', 'Incremental', 'Differential']:
        raise Exception('level inconnu')
    if job not in ['weekly', 'daily', 'monthly']:
        raise Exception ('job inconnu')
    try:
        day = int(day)
    except:
        raise Exception("Le jour n'est pas un entier ({0})".format(day))
    if not 0 < day < 8:
        raise Exception("Le jour {0} n'est pas correct n'est pas entre 1 et 7".format(day))
    if job == 'daily':
        if end_day == None:
            raise Exception('end_day doit être renseigné')
        try:
            end_day = int(end_day)
        except:
            raise Exception("Le jour de fin n'est pas un entier ({0})".format(end_day))
        if not 0 < end_day < 8:
            raise Exception("Le jour de fin {0} n'est pas correct n'est pas entre 1 et 7".format(end_day))
        if day == end_day:
            raise Exception('Jour de début et de fin identique, veuiller faire une sauvegarde hebdomadaire')
        if day > end_day:
            raise Exception('Le jour de fin est inférieur au jour de début')
    elif end_day != None:
        raise Exception('end_day ne concerne que les job daily')
    try:
        hour = int(hour)
    except:
        raise Exception("L'heure n'est pas un entier ({0})".format(hour))
    if not 0 <= hour < 24:
        raise Exception("L'heure {0} n'est pas correct entre 0 et 23".format(hour))
    jobs = load_bacula_jobs()

    for tjob in jobs:
        index = jobs.index(tjob)+1
        #test si le jour du job est déjà utilisé pour le même type de job
        if day == tjob['day'] and job== tjob['job']:
            raise_if_conflict(day, index)
        #si le nouveau job est de type daily, il ne faut pas de monthly ou
        #de weekly le même jour
        if job == 'daily':
            if tjob['job'] == 'weekly' and day == tjob['day']:
                raise_if_conflict(day, index)
            #si le job existant est du type daily, vérifier la plage et pas
            #seulement le jour
            if tjob['job'] == 'daily':
                if tjob['day'] < day <= tjob['end_day']:
                    raise_if_conflict(day, index)
        #si le job existant est de type daily, il faut verifié que le nouveau
        #job monthly ou weekly ne soit pas définit le même jour
        elif tjob['job'] == 'daily' and job == 'weekly':
            if tjob['day'] <= day <= tjob['end_day']:
                raise_if_conflict(day, index)

    retjob = {'job': job,
              'level': level,
              'day': day,
              'hour': hour}
    if end_day != None:
        retjob['end_day'] = end_day
    jobs.append(retjob)
    save_bacula_job(jobs)
    apply_bacula_jobs()

def del_job(jobnumber):
    if not bacula_active_dir():
        raise Disabled('Bacula dir désactivé')
    try:
        jobnumber = int(jobnumber)
    except ValueError:
        raise Exception("jobnumber n'est pas un nombre")
    jobs = load_bacula_jobs()
    jobs.pop(jobnumber)
    save_bacula_job(jobs)
    apply_bacula_jobs()

def apply_bacula_jobs(load=False):
    if not bacula_active_dir():
        if os.path.isfile(BACULA_CONFIG_JOB):
            os.unlink(BACULA_CONFIG_JOB)
        if os.path.isfile(BACULA_CONFIG_JOBSCHED):
            os.unlink(BACULA_CONFIG_JOBSCHED)
        raise Disabled('Bacula dir désactivé')
    def write_job(job, day, monthly=False):
        level = job['level']
        hour = job['hour']
        day = int(day)
        #weeknumber must be 1st for monthly jobs
        #weeknumber must be 2nd-5th if monthly job set same day
        if monthly:
            weeknumber = '1st '
        else:
            if str(day) in monthjobs:
                weeknumber = '2nd-5th '
            else:
                weeknumber = ''
        #if 12 < day < 24: day = yesterday
        #if 0 < day 12: day = today

        schehour = schedule['hour']
        if int(hour) > 12:
            if day == 1:
                newday = 7
            else:
                newday = day-1
        else:
            newday = day
            if schehour < hour:
                schehour = hour

        tday = DAY_TO_BACULA[newday]
        scheday = DAY_TO_BACULA[day]
        schemin = str(schedule['minute'])
        #time must be 4:01 not 4:1
        if len(schemin) == 1:
            schemin = '0{0}'.format(schemin)
        fhjob.write('Run = {0} {1}{2} at {3}:00\n'.format(level, weeknumber, tday, hour))
        fhjobpost.write('Run = Full {0}{1} at {2}:{3}\n'.format(weeknumber, scheday, schehour, schemin))
    jobs = load_bacula_jobs()
    if jobs == []:
        print "les programmations bacula ne sont pas configurées"
        _empty_file(BACULA_CONFIG_JOB)
        _empty_file(BACULA_CONFIG_JOBSCHED)
        return None
    fhjob = open(BACULA_CONFIG_JOB, 'w')
    fhjobpost = open(BACULA_CONFIG_JOBSCHED, 'w')

    #get all monthly job
    monthjobs = []
    for job in jobs:
        if job['job'] == 'monthly':
            monthjobs.append(str(job['day']))

    #load schedule
    schedule = load_schedule()
    for job in jobs:
        if job['job'] == 'daily':
            for i in range(int(job['day']), int(job['end_day'])+1):
                write_job(job, i)
        elif job['job'] == 'weekly':
            write_job(job, job['day'])
        elif job['job'] == 'monthly':
            write_job(job, job['day'], True)
        else:
            raise Exception('Type de job inconnu')
    fhjob.close()
    fhjobpost.close()

    if load:
        reload_conf(test='dir')

def display_bacula_jobs():
    if not bacula_active_dir():
        raise Disabled('Bacula dir désactivé')
    jobs = load_bacula_jobs()
    if jobs == []:
        return None
    disp = []
    for save in jobs:
        jobnumber = jobs.index(save)
        day = save['day']
        level = LEVEL_TO_FR[save['level']]
        hour = save['hour']
        job = save['job']
        message = "{0} : Sauvegarde {1} ".format(jobnumber+1, level.lower())
        if job == 'daily':
            end_day = save['end_day']
            message += "de la nuit du {0} à la nuit du {1} ".format(DAY_TO_STRING[int(day)], DAY_TO_STRING[int(end_day)])
        elif job == 'weekly':
            message += "dans la nuit du {0} ".format(DAY_TO_STRING[int(day)])
        else:
            message += "dans la première nuit du mois du {0} ".format(DAY_TO_STRING[int(day)])
        message += "à {0}:00".format(hour)
        disp.append((jobnumber, message))
    return disp

