# -*- coding: utf-8 -*-

import sys
from pyeole.process import system_code, system_out
from pyeole.service import service_out
from pyeole.bacula import BACULA_SUPPORT
from os import system, unlink
from creole.parsedico import parse_dico
from os.path import isfile
from tempfile import mkstemp
from shutil import move

MSG = """La restauration est lancée en tâche de fond.
Vous pouvez suivre son évolution dans le fichier /var/log/bacula/restore.txt"""

catalog_pool_name = 'scribe-dir-catalog-0003'
catalog_db_file = '/var/lib/bacula/bacula.db'
catalog_sql_file = '/var/lib/bacula/bacula.sql'

dico = None

def load_dico():
    global dico
    if dico == None:
        dico = parse_dico()

def bacula_active():
    load_dico()
    return dico['activer_bacula'] == 'oui' and dico['activer_bacula_dir'] == 'oui'

def bacula_restore_one_file(filename, jobid):
    bconsole_command("file={0} done\n{1}\n".format(filename, jobid))
    print MSG

def bacula_restore_all_files(jobid):
    bconsole_command('\ndone\n{0}\n'.format(jobid), prefix='select current all yes')
    print MSG

def bacula_delete_catalog():
    service_out('bacula-director', 'stop')
    unlink(catalog_db_file)

def bacula_restore_dump(filename=catalog_sql_file):
    if isfile(catalog_db_file):
        print "Le catalogue existe déjà"
        sys.exit(1)
    system('/usr/bin/sqlite3 {0} < {1}'.format(catalog_db_file, filename))
    system_code(['chown', 'bacula:root', catalog_db_file])
    service_out('bacula-director', 'start')

def bacula_search(filenames):
    if type(filenames) != list or filenames == []:
        raise Exception('Il faut une liste de fichier pour bacula_search')
    cmd = '\n@output\nfind'
    for filename in filenames:
        cmd += ' {0}'.format(filename)
    bconsole_command(cmd)

def bconsole_command(cmd, prefix='select current yes'):
    if not bacula_active():
        raise Exception("Bacula n'est pas actif, impossible de restaurer")
    bacula_fd = dico['bacula_fd_name']
    code = system_code(["bconsole", "-c", "/etc/bacula/bconsole.conf"], stdin="restore FileSet=FileSetSauvegarde Client={0} {1} {2}\n".format(bacula_fd, prefix, cmd))
    if code != 0:
        raise Exception('Erreur bconsole')

def extract_bacula_files():
    load_dico()
    try:
        extract_file(catalog_sql_file)
    except Exception, e:
        print e
        sys.exit(1)
    try:
        extract_file('/var/sauvegardes/{0}-JobDefsSauvegarde.bsr'.format(dico['bacula_dir_name']))
    except Exception, e:
        print e
        sys.exit(1)
    try:
        extract_file('/var/sauvegardes/{0}-JobDefsCatalog.bsr'.format(dico['bacula_dir_name']))
    except Exception, e:
        print e
        sys.exit(1)
    try:
        extract_file(BACULA_SUPPORT)
    except Exception, e:
        print e
    try:
        extract_file('/var/lib/eole/config/baculamail.conf')
    except Exception, e:
        print e
    try:
        extract_file('/var/lib/eole/config/baculajobs.conf')
    except Exception, e:
        print e

def extract_file(filename, dest_file=None, volumename=catalog_pool_name):
    """Extract file directly in a volume"""
    tmp_file = mkstemp()[1]
    include_file = open(tmp_file, 'w')
    include_file.write(filename)
    include_file.close()
    extract_dir = '/tmp'
    code, stdout, stderr = system_out(['/usr/sbin/bls', '-V', volumename, 'FileStorage', '-i', tmp_file])
    if code == 1:
        unlink(tmp_file)
        sys.exit(1)
    if stdout.strip().split('\n')[-1] == "0 files found.":
        print "Pas de fichier {0} dans le volume {1}".format(filename, volumename)
        raise Exception("Impossible de lister le catalogue {0}".format(volumename))
    code, stdout, stderr = system_out(['/usr/sbin/bextract', '-V', volumename, 'FileStorage', extract_dir, '-i', tmp_file])
    if code == 1:
        unlink(tmp_file)
        print "Impossible de restaurer {0} du catalogue {1}".format(filename, volumename)
        print stderr
        sys.exit(1)
    unlink(tmp_file)
    if dest_file == None:
        dest_file = filename
    move("{0}{1}".format(extract_dir, filename), dest_file)

def exit_if_running_jobs():
    if not bacula_active():
        raise Exception("Bacula n'est pas actif, impossible de restaurer")
    code, out, err = system_out(["/usr/bin/bconsole"], stdin='status dir')
    try:
        running_lines = out.split('\n====\n')[1]
    except:
        print "Erreur de status"
        print out, err
        sys.exit(1)

    if not 'No Jobs running.' in running_lines and not 'Pas de job en cours.' in running_lines:
        print "Impossible de restaurer, jobs en cours"
        print "\n".join(running_lines.split('\n')[5:])
        sys.exit(1)

