# -*- coding: UTF-8 -*-
"""
utilitaires de lancement de processus
"""
import sys
import subprocess
from os.path import isfile
from creole import config
from pyeole.deprecation import deprecated

@deprecated
def run_subprocess(cmd, stdout=True, exit=False, container=None, context=False, stdin=None, stderr=True, code=False):
    """Raccourci d'execution pour l'execution d'une commande shell avec capture de STDOUT
    Attention, cmd est une liste :
    cmd='ls -al' s'ecrit cmd=['ls','-la']
    """
    ret = False
    if not stdout:
        if exit:
            # Pas possible de verifier le code retour si on retour
            ret = False
        else:
            ret = True
        dstdout = subprocess.PIPE
    else:
        dstdout = sys.stdout

    if stderr:
        dstderr = sys.stderr
    else:
        dstderr = subprocess.PIPE

    if container != None:
        if container != config.VIRTMASTER:
            cmd = _gen_container_cmd(cmd, container, context)
    elif context != False:
        raise Exception('Si container == None, context doit être égal à False')
    process = subprocess.Popen(cmd, stdout=dstdout, stderr=dstderr, stdin=subprocess.PIPE)
    if ret == True:
        return process.stdout
    process.communicate(stdin)
    returncode = process.returncode
    if exit and returncode:
        raise Exception('La commande "%s" a echoué' % str(" ".join(cmd)))
    if code:
        #si code est True, alors le code de retour est renvoyé
        return returncode

def _gen_container_cmd(cmd, container, context, env=None, pty=False):
    """
    génération de la commande en mode conteneur
    """
    if isfile(config.containers_file):
        dico_container = {}
        execfile(config.containers_file, {}, dico_container)
        if dico_container.has_key('container_path_%s' % container):
            if context == False:
                path = dico_container['container_path_%s' % container]
                if path != "":
                    cmd = ['chroot', path] + cmd
            else:
                ip_address = dico_container['container_ip_%s' % container]
                if ip_address != '127.0.0.1':
                    if not test_conteneur(container, ip_address):
                        raise Exception('Conteneur {0} inaccessible'.format(container))
                    escaped_cmd = []
                    if env != None:
                        for key, value in env.items():
                            escaped_cmd.append('%s=%s'%(key, value))
                    # échappement des doubles-quotes et des antislashes pour #757 et #1439
                    for elt in cmd:
                        escaped_cmd.append('"%s"' % elt.replace('\\', '\\\\').replace('"', '\\"'))
                    cmd = ['/usr/bin/ssh']
                    if pty:
                        cmd.append('-t')
                    cmd.extend(['-q', '-o', 'LogLevel=ERROR', '-o', 'StrictHostKeyChecking=no',
                                'root@%s' % ip_address])
                    cmd.extend(escaped_cmd)
    return cmd

def system_code(cmd, stdin=None, container=config.VIRTMASTER, context=True, env=None, pty=False):
    """
    exécution d'une commande shell :
    - affichage des messages (stdout/stderr) "à l'écran"
    - retourne le code de retour sous la forme d'un entier
    @cmd : commande sous la forme d'une liste
    @stdin : chaine à passer à stdin ou None
    @container : conteneur dans lequel exécuter la commande
    @context : conservation du contexte (dans un conteneur)
    """
    # fix pour éviter "IOError: [Errno 9] Bad file descriptor" (EAD)
    if sys.stdout.mode == 'wb':
        print "Sortie binaire détectée => affichage masqué"
        return system_out(cmd, stdin=stdin, container=container, context=context)[0]
    if container != config.VIRTMASTER:
        cmd = _gen_container_cmd(cmd, container, context, env, pty)
    if pty:
        import pty
        if stdin != None:
            raise Exception('Utilisation de pty et stdin ensemble est impossible')
        master, slave = pty.openpty()
        process = subprocess.Popen(cmd, stdin=master, stderr=slave, env=env)
        process.communicate()
    else:
        process = subprocess.Popen(cmd, stdin=subprocess.PIPE, env=env)
        process.communicate(stdin)
    return process.returncode

def system_out(cmd, stdin=None, container=config.VIRTMASTER, context=True, env=None, pty=False):
    """
    exécution d'une commande shell :
    - retourne un tuple contenant :
        - le code de retour sous la forme d'un entier
        - stdout
        - stderr
    @cmd : commande sous la forme d'une liste
    @stdin : chaine à passer à stdin ou None
    @container : conteneur dans lequel exécuter la commande
    @context : conservation du contexte (dans un conteneur)
    """
    if container != config.VIRTMASTER:
        cmd = _gen_container_cmd(cmd, container, context, env, pty)
    if pty:
        import pty
        if stdin != None:
            raise Exception('Utilisation de pty et stdin ensemble est impossible')
        master, slave = pty.openpty()
        process = subprocess.Popen(cmd, stdin=master,
                               stdout=subprocess.PIPE,
                               stderr=slave,
                               env=env)
        stdout, stderr = process.communicate()
    else:
        process = subprocess.Popen(cmd, stdin=subprocess.PIPE,
                               stdout=subprocess.PIPE,
                               stderr=subprocess.PIPE,
                               env=env)
        stdout, stderr = process.communicate(stdin)
    return (process.returncode, stdout, stderr)

def tcpcheck(ip, port, timeout=1):
    cmd = ['/usr/bin/tcpcheck', str(timeout), '{0}:{1}'.format(ip, port)]
    return system_out(cmd)[1].strip().lower().endswith('alive')

def test_conteneur(container, ip_address=None):
    if container == config.VIRTMASTER:
        return True
    dico_container = {}
    execfile(config.containers_file, {}, dico_container)
    try:
        path = dico_container['container_path_%s' % container]
    except:
        print "Conteneur {0} inconnu".format(container)
        return False
    if path == '':
        return True
    splitpath = path.split('/')
    groupname = splitpath[len(splitpath)-2]
    cmd = ['/usr/bin/lxc-info', '-n', groupname]
    if not system_out(cmd)[1].strip().lower().endswith(' is running'):
        return False
    if ip_address == None:
        ip_address = dico_container['container_ip_%s' % container]
    if not tcpcheck(ip_address, '22', '2'):
        return False
    return True
