# -*- coding: UTF-8 -*-
from time import sleep
from os.path import join, isfile
from pyeole.process import run_subprocess, system_out, system_code
from creole.config import init_services_file, VIRTMASTER, containers_file
try:
    import json
except:
    import simplejson as json

def service(service, action, container, stdout=True, stderr=True):
    cmds = _gen_service_cmd(service, action, container)
    for cmd in cmds:
        run_subprocess(cmd, exit=False, container=container, context=True,
                       stdout=stdout, stderr=stderr)

def _gen_service_cmd(service, action, container):
    """
    génération de la commande en fonction des services
    """
    cmds = []
    # fix pour les scripts ne supportant pas restart
    if isfile(init_services_file):
        dico_init = json.load(file(init_services_file))
    else:
        dico_init = {}
    init, pty = dico_init.get(service, ('service', False))
    if action == 'restart':
        if init == 'upstart':
            cmds.append(['/sbin/restart-wrapper', service])
            #res = system_out(['status', service], container=container)
            #if "%s stop/waiting\n" % service == res[1]:
            #    cmds.append(['/sbin/start', service])
            #else:
            #    cmds.append(['/sbin/restart', service])
        elif init == 'initd':
            cmds.append(['/etc/init.d/%s' % service, 'stop'])
            cmds.append(['sleep', '1'])
            cmds.append(['/etc/init.d/%s' % service, 'restart'])
        else:
            cmds.append(['/usr/sbin/service', service, "stop"])
            cmds.append(['sleep', '1'])
            cmds.append(['/usr/sbin/service', service, "start"])
    else:
        if init == 'initd':
            cmds.append(['/etc/init.d/%s' % service, action])
        else:
            cmds.append(['/usr/sbin/service', service, action])
    return cmds, pty

def service_code(service, action, container=VIRTMASTER):
    """
    action sur un service :
    - affichage des messages (stdout/stderr) "à l'écran"
    - retourne le *dernier* code de retour sous la forme d'un entier
    @service : nom du service
    @action : action sur le service (stop, start, restart)
    @container : conteneur du service
    """
    cmds, use_pty = _gen_service_cmd(service, action, container)
    for cmd in cmds:
        if cmd[0] == 'sleep':
            ret = sleep(int(cmd[1]))
        else:
            env = None
            pty = False
            if use_pty:
                dico_container = {}
                execfile(containers_file, {}, dico_container)
                path = dico_container.get('container_path_%s' % container, '')
                try:
                    import sys, fcntl, termios, struct
                    data = fcntl.ioctl(sys.stdout.fileno(), termios.TIOCGWINSZ, '1234')
                    cols = struct.unpack('hh', data)[1]
                    ti_file = '/tmp/eole.ti'
                    fh = open(ti_file, 'w')
                    fh.write('eole,\n\txenl,\n\thpa=\E[%i%p1%dG,\n\tsetaf=\E[3%p1%dm,\n\tcols#{0},\n\top=\E[39;49m,\n'.format(cols))
                    fh.close()
                    terminfo_dir = join("/", path, 'etc/terminfo/')
                    ticcmd = ['/usr/bin/tic', '-o', terminfo_dir, ti_file]
                    system_code(ticcmd)
                    env = {'TERM': 'eole'}
                    pty = True
                except:
                    pty = False
                    env = None
            ret = system_code(cmd, container=container, env=env, pty=pty)
    return ret

def service_out(service, action, container=VIRTMASTER):
    """
    action sur un service :
    - retourne un tuple contenant (pour la *dernière* commande) :
        - le code de retour sous la forme d'un entier
        - stdout
        - stderr
    @service : nom du service
    @action : action sur le service (stop, start, restart)
    @container : conteneur du service
    """
    cmds, pty = _gen_service_cmd(service, action, container)
    for cmd in cmds:
        if cmd[0] == 'sleep':
            ret = sleep(int(cmd[1]))
        else:
            #no env for system_out
            ret = system_out(cmd, container=container)
    return ret
